/*
** Module   : XEntityImpl
** Abstract : DOM and SAX common implementation.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130204 Created initial version. Major rework to use w3c backend.
** 002 EVL 20130214 Adding the base inheritance from XCommonImpl class to place the XML related
**                  common features.  
** 003 CA  20130221 Added UNIQUE-ID and NAMESPACE-URI support. 
** 004 EVL 20130222 Adding SUBTYPE attribute and INSERT-BEFORE method support.
** 005 OM  20130304 Refactored isValid and isUnknown of WrappedResource to valid and unknown.
*/

package com.goldencode.p2j.xml;

import java.util.*;
import org.w3c.dom.*;
import com.goldencode.p2j.util.*;

/**
 * Implementation of the common features for DOM.
 */
public abstract class XEntityImpl
extends XCommonImpl
implements XEntity
{
   /** The handle associated with this object. */
   private handle hXEntity = null;

   /** Element associates with current XEntity class. */
   private Element xnode = null;//TODO maybe Node is more usable here
   
   /**
    * Default constructor.
    */
   public XEntityImpl()
   {
      hXEntity = new handle();
   }

   /**
    * Special constructor accepting the support object.
    * 
    * @param    value
    *           The object to be associated with the given handle object.
    */
   public XEntityImpl(WrappedResource value)
   {
      this();
      hXEntity.assign(value);
   }

   /**
    * Special constructor accepting a X-Document or X-Node-Reference object.
    * 
    * @param    element
    *           The DOM XML Element object to be associated with the given
    *           handle object.
    */
   public XEntityImpl(Element element)
   {
      this();
      xnode = element;
      hXEntity.assign(this);
   }

   /**
    * Getting the Element backend object for this XEntity.
    * 
    * @return   The Element associated with object.
    */
   public Element getNode()
   {
      return xnode;
   }
   
   /**
    * Getting the handle of the object.  
    *
    * @return   Internal handle object associated with this node.
    */
   @Override
   public handle getHandle()
   {
      return hXEntity;
   }
   
   /**
    * Reports if this object is valid for use.  
    *
    * @return   <code>true</code> if we are valid (can be used).
    */
   @Override
   public boolean valid()
   {
      return super.valid() && hXEntity != null && xnode != null;
   }
   
   /**
    * Reports if this object is unknown.  
    *
    * @return   <code>true</code> if object is unknown.
    */
   @Override
   public boolean unknown()
   {
      return super.unknown() || hXEntity == null || xnode == null;
   }

   /**
    * Gets the the subtype of the object.
    * 
    * @return   The character value of the subtype attribute.
    */
   @Override
   public character getSubType()
   {
      return XmlFactory.convertNodeToAbl(xnode.getNodeType());
   }
   
   /**
    * Sets the the subtype of the object.
    * 
    * @param    value
    *           The new value of the subtype object attribute.
    */
   @Override
   public void setSubType(character value)
   {
      readOnlyError("SUBTYPE");
   }

   /**
    * Get the type of this handle.
    *
    * @return See above.
    */
   @Override
   public character getResourceType()
   {
      return new character(getType());
   }

   protected abstract String getType();//TODO really don't sure about this method

   /**
    * Sets the the subtype of the object.
    * 
    * @param    value
    *           The new value of the subtype object attribute.
    */
   @Override
   public void setSubType(String value)
   {
      setSubType(new character(value));
   }

   /**
    * Inserts a node as a child of this document before another node or last if other node is
    * unknown.
    *
    * @param    handle1
    *           The handle representing the child to insert.
    * @param    handle2
    *           The handle representing the XML node to be inserted before.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical insertBefore(handle handle1, handle handle2)
   {
      if (handle2 == null || handle2.isUnknown())
      {
         return appendChild(handle1);
      }
      final Element node2 = ((XEntityImpl) handle2.get()).getNode();
      final Element node1 = ((XEntityImpl) handle1.get()).getNode();
      final Node result = xnode.insertBefore(node1, node2);

      return new logical(result != null);
   }

   /**
    * Gets the the number of children objects for the current one.
    * 
    * @return   The integer number of the children.
    */
   @Override
   public integer getNumChildren()
   {
      return new integer(xnode.getChildNodes().getLength());
   }

   /**
    * Removing the specified child from the node.
    *
    * @param newChildHandle The handle class instance that represents the node to insert in the
    *                       tree.
    * @param oldChildHandle The handle class instance that represents the node to remove from the
    *                       tree.
    *
    * @return The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical replaceChild(handle newChildHandle, handle oldChildHandle)
   {
      if (!newChildHandle._isValid() || !oldChildHandle._isValid())
      {
         return new logical(false);
      }
      final Node newChild = ((XEntityImpl) newChildHandle.get()).getNode();
      final Node oldChild = ((XEntityImpl) oldChildHandle.get()).getNode();

      final Node node = xnode.replaceChild(newChild, oldChild);
      // TODO i think node is always not null

      return new logical(node != null);
   }

   @Override
   public integer getChildNum()
   {
      final Node parent = xnode.getParentNode();
      if (parent == null)
      {
         return new integer();
      }

      final NodeList childNodes = parent.getChildNodes();
      for (int i = 0 ; i < childNodes.getLength(); i++)
      {
         if (xnode.equals(childNodes.item(i)))
         {
            return new integer(i + 1);
         }
      }

      return new integer();  //TODO autogenerated code by evk, please fix this
   }

   /**   TODO
    * Get the <code>name</code> attribute of handle.
    *
    * @return See above.
    */
   @Override
   public character getName()
   {
      return new character(xnode.getNodeName());
   }

   /**       TODO
    * Sets the name of this handle.This method is outside interfaces and only meant for
    * initializing this within the implementation.
    *
    * @param name The handle type to be set.
    */
   @Override
   public void setName(character name)
   {
      xnode.setAttribute("name", name.toStringMessage());
      super.setName(name);
   }

   /**
    * Appending the new child to the current object.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be appended.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   @Override
   public logical appendChild(handle hChild)
   {
      return appendChild(hChild, (character) null, (character) null);
   }

   /** TODO why 3 parameters ?
    * Appending the new child to the current object.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be appended.
    * @param    name
    *           The name of the child to be appended.
    * @param    type
    *           The type of the child to be appended.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   @Override
   public logical appendChild(handle hChild, character name, character type)
   {
      Element elChild = ((XEntityImpl) hChild.get()).getNode();
      if (xnode.appendChild(elChild) != null)
      {
         return new logical(true);
      }
      else
      {
         return new logical(false);
      }
   }

   /**
    * Getting the handle object of the child node with specified index.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to put
    *           the results.
    * @param    index
    *           1-based index of the child node to be retrieved.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical getChild(handle hChild, int index)
   {
      return getChild(hChild, new integer(index));
   }
   
   /**
    * Getting the handle object of the child node with specified index.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to put
    *           the results.
    * @param    index
    *           1-based index of the child node to be retrieved.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   @Override
   public logical getChild(handle hChild, integer index)
   {
      final int ndx = index.intValue();

      final NodeList nlst = xnode.getChildNodes();
      
      if (ndx < 1 || ndx > nlst.getLength())
      {
         return new logical(false);
      }
      
      final Element elem = (Element)nlst.item(ndx - 1);
      hChild.assign(new XNodeRefImpl(elem));

      return new logical(true);
   }
   
   /**
    * Removing the specified child from the node.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be removed.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical removeChild(handle hChild)
   {
      if (!hChild._isValid())
      {
         return new logical(false);
      }

      final Element elChild = ((XEntityImpl) hChild.get()).getNode();
      final boolean result = xnode.removeChild(elChild) != null;
      
      return new logical(result);
   }

   /**
    * Clean up the current node.
    * 
    */
   public void clear()
   {
      hXEntity = null;
      xnode.getOwnerDocument().removeChild(xnode);
      xnode = null;
   }

   /**
    * Gets the the unique ID number associated to this object by the underlying system. Not the 
    * same as the handle value itself.
    * 
    * @return   The integer number of the children.
    */
   @Override
   public integer getUniqueID()
   {
      // TODO Auto-generated method stub
      return null;
   }
   /**
    * Returns the value of the the NAMESPACE-URI attribute.
    *
    * @return    See above.
    */
   @Override
   public character getNamespaceURI()
   {
      final String namespaceUri = xnode.getNamespaceURI();

      if (namespaceUri != null)
      {
         return new character(namespaceUri);
      }
      else
      {
         return new character();
      }
   }

   /**
    * Setter for NAMESPACE-URI attribute. Calls {@link #readOnlyError}, as this attribute is r/o
    * for this resource.
    *
    * @param    uri
    *           Not used.
    */
   @Override
   public void setNamespaceURI(String uri)
   {
      readOnlyError("NAMESPACE-URI");
   }

   /**
    * Setter for NAMESPACE-URI attribute. Calls {@link #readOnlyError}, as this attribute is r/o
    * for this resource.
    *
    * @param    uri
    *           Not used.
    */
   @Override
   public void setNamespaceURI(character uri)
   {
      readOnlyError("NAMESPACE-URI");
   }


   /**
    * Removes the specified attribute.
    *
    * @param    name
    *           The name of the attribute to remove.
    * @param    uri
    *           The URI of the attribute to remove.
    *
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   @Override
   public logical removeAttribute(character name, character uri)
   {
      final boolean result;
      final Element node = getNode();
      final String nameStr = name.toStringMessage();
      if (uri.isUnknown())
      {
         result = node.hasAttribute(nameStr);
         node.removeAttribute(nameStr);
      }
      else
      {
         result = node.hasAttributeNS(uri.toStringMessage(), nameStr);
         node.removeAttributeNS(uri.toStringMessage(), nameStr);
      }
      return new logical(result);
   }



   /**
    * Returns the value of the the NAMESPACE-PREFIX attribute.
    *
    * @return    See above.
    */
   @Override
   public character getNamespacePrefix()
   {
      //TODO
      return null;
   }
}
