/*
** Module   : XmlFactory
** Abstract : DOM and SAX XML instance creation.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 EVL 20130204 Created initial version. Major rework to use w3c backend.
** 002 EVL 20130208 Added SAX XML Support.
** 003 CA  20130308 The CREATE APIs need to receive a handle parameter, to properly initiate it,
**                  as returning the new handle instance and changing the handle reference in the
**                  caller is not the right approach.
** 004 CA  20130309 Added missing APIs related to WIDGET-POOL clause.
** 005 CA  20130311 Added more missing APIs related to WIDGET-POOL clause.
*/

package com.goldencode.p2j.xml;

import org.w3c.dom.*;
import com.goldencode.p2j.util.*;

/**
 * The set of methods to work with DOM XML and SAX support implementation in P2J.
 */
public class XmlFactory
{
   /**
    * Constructs a new instance of the Progress X-Document object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    */
   public static void createXDocument(handle h)
   {
      h.assign(new XDocumentImpl());
   }

   /**
    * Constructs a new instance of the Progress X-Document object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    poolName
    *           The name of the pool to which the resource will be added
    */
   public static void createXDocument(handle h, String poolName)
   {
      createXDocument(h, new character(poolName));
   }

   /**
    * Constructs a new instance of the Progress X-Document object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    poolName
    *           The name of the pool to which the resource will be added
    */
   public static void createXDocument(handle h, character poolName)
   {
      h.assign(new XDocumentImpl());
      // TODO: add it to the pool
   }

   /**
    * Constructs a new instance of the Progress X-Node-Reference object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    */
   public static void createXNodeReference(handle h)
   {
      h.assign(new XNodeRefImpl());
   }

   /**
    * Constructs a new instance of the Progress X-Node-Reference object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    poolName
    *           The name of the pool to which the resource will be added
    */
   public static void createXNodeReference(handle h, String poolName)
   {
      createXNodeReference(h, new character(poolName));
   }

   /**
    * Constructs a new instance of the Progress X-Node-Reference object and
    * retrieves the handle variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    poolName
    *           The name of the pool to which the resource will be added
    */
   public static void createXNodeReference(handle h, character poolName)
   {
      h.assign(new XNodeRefImpl());
      // TODO: add it to the pool
   }

   /**
    * Constructs a new instance of the Progress X-Node-Reference object and
    * retrieves the handle variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    doc
    *           The X-Document object to create X-Node under.
    * @param    name
    *           The name of the node to create.
    * @param    type
    *           The type of the node to create.
    */
   public static void createXNodeReference(handle h,
                                           Document doc,
                                           character name,
                                           character type)
   {
      String nameImpl = null;
      
      if (!name.isUnknown())
      {
         nameImpl = name.toStringMessage();
      }

      Element xelement = doc.createElement(nameImpl);

      h.assign(new XNodeRefImpl(xelement));
   }

   /**
    * Constructs a new instance of the Progress X-Node-Reference object and
    * retrieves the handle variable associated with it. The approach is
    * slightly different. Each node will have additional paranmeter - Name
    * Space and node will be addressed by namespace and name pair instead
    * of the simple name.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    doc
    *           The X-Document object to create X-Node under.
    * @param    nsUri
    *           Name space associated with the current set of names.
    * @param    name
    *           The name of the node to create.
    * @param    type
    *           The type of the node to create.
    */
   public static void createXNodeReference(handle    h,
                                           Document  doc,
                                           character nsUri,
                                           character name,
                                           character type)
   {
      String nameImpl = null;
      String nsUriImpl = null;
      
      if (!nsUri.isUnknown())
      {
         nsUriImpl = nsUri.toStringMessage();
      }
      if (!name.isUnknown())
      {
         nameImpl = name.toStringMessage();
      }
      
      Element xelement = doc.createElementNS(nsUriImpl, nameImpl);

      h.assign(new XNodeRefImpl(xelement)); 
   }

   /**
    * Constructs a new instance of the Progress Sax-Attributes object and retrieves the handle
    * variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    */
   public static void createSaxAttributes(handle h)
   {
      createSaxAttributes(h, new character());
   }

   /**
    * Constructs a new instance of the Progress Sax-Attributes object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The widget pool in which the object handle is created.
    */
   public static void createSaxAttributes(handle h, String widgetPool)
   {
      createSaxAttributes(h, new character(widgetPool));
   }

   /**
    * Constructs a new instance of the Progress Sax-Attributes object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The widget pool in which the object handle is created.
    */
   public static void createSaxAttributes(handle h, character widgetPool)
   {
      // TODO: implement this
   }
   
   /**
    * Constructs a new instance of the Progress Sax-Reader object and retrieves the handle
    * variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    */
   public static void createSaxReader(handle h)
   {
      createSaxReader(h, new character());
   }

   /**
    * Constructs a new instance of the Progress Sax-Reader object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The widget pool in which the object handle is created.
    */
   public static void createSaxReader(handle h, String widgetPool)
   {
      createSaxReader(h, new character(widgetPool));
   }

   /**
    * Constructs a new instance of the Progress Sax-Reader object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The widget pool in which the object handle is created.
    */
   public static void createSaxReader(handle h, character widgetPool)
   {
      // TODO: implement this
   }
   
   /**
    * Constructs a new instance of the Progress Sax-Writer object and retrieves the handle
    * variable associated with it.
    * 
    * @param    h
    *           The handle where the new resource will be set as the referent.
    */
   public static void createSaxWriter(handle h)
   {
      createSaxWriter(h, new character());
   }

   /**
    * Constructs a new instance of the Progress Sax-Writer object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The widget pool in which the object handle is created.
    */
   public static void createSaxWriter(handle h, String widgetPool)
   {
      createSaxWriter(h, new character(widgetPool));
   }

   /**
    * Constructs a new instance of the Progress Sax-Writer object and retrieves the handle
    * variable associated with it.
    *
    * @param    h
    *           The handle where the new resource will be set as the referent.
    * @param    widgetPool
    *           The widget pool in which the object handle is created.
    */
   public static void createSaxWriter(handle h, character widgetPool)
   {
      // TODO: implement this
   }


   public static character convertNodeToAbl(short nodeId)
   {
      switch (nodeId)
      {
         case Node.ATTRIBUTE_NODE:
            return new character("ATTRIBUTE");
         case Node.CDATA_SECTION_NODE:
            return new character("CDATA_SECTION");
         case Node.COMMENT_NODE:
            return new character("COMMENT");
         case Node.DOCUMENT_NODE:
            return new character("DOCUMENT");
         case Node.DOCUMENT_FRAGMENT_NODE:
            return new character("DOCUMENT_FRAGMENT");
         case Node.ELEMENT_NODE:
            return new character("ELEMENT");
         case Node.ENTITY_REFERENCE_NODE:
            return new character("ENTITY_REFERENCE");
         case Node.PROCESSING_INSTRUCTION_NODE:
            return new character("PROCESSING_INSTRUCTION");
         case Node.TEXT_NODE:
            return new character("TEXT");
      }
      return new character();
   }
}
